from __future__ import unicode_literals, absolute_import
import io
import os
import re
import abc
import csv
import sys
import zipp
import operator
import functools
import itertools
import collections
from ._compat import install, NullFinder, ConfigParser, suppress, map, FileNotFoundError, IsADirectoryError, NotADirectoryError, PermissionError, pathlib, PYPY_OPEN_BUG, ModuleNotFoundError, MetaPathFinder, email_message_from_string
from importlib import import_module
from itertools import starmap
__metaclass__ = type
__all__ = [u'Distribution', u'PackageNotFoundError', u'distribution',
    u'distributions', u'entry_points', u'files', u'metadata', u'requires',
    u'version']


class PackageNotFoundError(ModuleNotFoundError):
    u'The package was not found.'


class EntryPoint(collections.namedtuple(u'EntryPointBase', u'name value group')
    ):
    u'An entry point as defined by Python packaging conventions.\n\n    See `the packaging docs on entry points\n    <https://packaging.python.org/specifications/entry-points/>`_\n    for more information.\n    '
    pattern = re.compile(
        u'(?P<module>[\\w.]+)\\s*(:\\s*(?P<attr>[\\w.]+))?\\s*(?P<extras>\\[.*\\])?\\s*$'
        )
    u"\n    A regular expression describing the syntax for an entry point,\n    which might look like:\n\n        - module\n        - package.module\n        - package.module:attribute\n        - package.module:object.attribute\n        - package.module:attr [extra1, extra2]\n\n    Other combinations are possible as well.\n\n    The expression is lenient about whitespace around the ':',\n    following the attr, and following any extras.\n    "

    def load(self):
        u'Load the entry point from its definition. If only a module\n        is indicated by the value, return that module. Otherwise,\n        return the named object.\n        '
        match = self.pattern.match(self.value)
        module = import_module(match.group(u'module'))
        attrs = filter(None, (match.group(u'attr') or u'').split(u'.'))
        return functools.reduce(getattr, attrs, module)

    @property
    def extras(self):
        match = self.pattern.match(self.value)
        return list(re.finditer(u'\\w+', match.group(u'extras') or u''))

    @classmethod
    def _from_config(cls, config):
        return [cls(name, value, group) for group in config.sections() for 
            name, value in config.items(group)]

    @classmethod
    def _from_text(cls, text):
        config = ConfigParser(delimiters=u'=')
        config.optionxform = str
        try:
            config.read_string(text)
        except AttributeError:
            config.readfp(io.StringIO(text))
        return EntryPoint._from_config(config)

    def __iter__(self):
        u'\n        Supply iter so one may construct dicts of EntryPoints easily.\n        '
        return iter((self.name, self))


class PackagePath(pathlib.PurePosixPath):
    u'A reference to a path in a package'

    def read_text(self, encoding=u'utf-8'):
        with self.locate().open(encoding=encoding) as stream:
            return stream.read()

    def read_binary(self):
        with self.locate().open(u'rb') as stream:
            return stream.read()

    def locate(self):
        u'Return a path-like object for this path'
        return self.dist.locate_file(self)


class FileHash:

    def __init__(self, spec):
        self.mode, _, self.value = spec.partition(u'=')

    def __repr__(self):
        return u'<FileHash mode: {} value: {}>'.format(self.mode, self.value)


class Distribution:
    u'A Python distribution package.'

    @abc.abstractmethod
    def read_text(self, filename):
        u'Attempt to load metadata file given by the name.\n\n        :param filename: The name of the file in the distribution info.\n        :return: The text if found, otherwise None.\n        '

    @abc.abstractmethod
    def locate_file(self, path):
        u'\n        Given a path to a file in this distribution, return a path\n        to it.\n        '

    @classmethod
    def from_name(cls, name):
        u"Return the Distribution for the given package name.\n\n        :param name: The name of the distribution package to search for.\n        :return: The Distribution instance (or subclass thereof) for the named\n            package, if found.\n        :raises PackageNotFoundError: When the named package's distribution\n            metadata cannot be found.\n        "
        for resolver in cls._discover_resolvers():
            dists = resolver(name)
            dist = next(dists, None)
            if dist is not None:
                return dist
        else:
            raise PackageNotFoundError(name)

    @classmethod
    def discover(cls):
        u'Return an iterable of Distribution objects for all packages.\n\n        :return: Iterable of Distribution objects for all packages.\n        '
        return itertools.chain.from_iterable(resolver() for resolver in cls
            ._discover_resolvers())

    @staticmethod
    def _discover_resolvers():
        u'Search the meta_path for resolvers.'
        declared = (getattr(finder, u'find_distributions', None) for finder in
            sys.meta_path)
        return filter(None, declared)

    @property
    def metadata(self):
        u'Return the parsed metadata for this Distribution.\n\n        The returned object will have keys that name the various bits of\n        metadata.  See PEP 566 for details.\n        '
        text = self.read_text(u'METADATA') or self.read_text(u'PKG-INFO'
            ) or self.read_text(u'')
        return email_message_from_string(text)

    @property
    def version(self):
        u"Return the 'Version' metadata for the distribution package."
        return self.metadata[u'Version']

    @property
    def entry_points(self):
        return EntryPoint._from_text(self.read_text(u'entry_points.txt'))

    @property
    def files(self):
        file_lines = self._read_files_distinfo() or self._read_files_egginfo()

        def make_file(name, hash=None, size_str=None):
            result = PackagePath(name)
            result.hash = FileHash(hash) if hash else None
            result.size = int(size_str) if size_str else None
            result.dist = self
            return result
        return file_lines and starmap(make_file, csv.reader(file_lines))

    def _read_files_distinfo(self):
        u'\n        Read the lines of RECORD\n        '
        text = self.read_text(u'RECORD')
        return text and text.splitlines()

    def _read_files_egginfo(self):
        u'\n        SOURCES.txt might contain literal commas, so wrap each line\n        in quotes.\n        '
        text = self.read_text(u'SOURCES.txt')
        return text and map(u'"{}"'.format, text.splitlines())

    @property
    def requires(self):
        u'Generated requirements specified for this Distribution'
        return self._read_dist_info_reqs() or self._read_egg_info_reqs()

    def _read_dist_info_reqs(self):
        spec = self.metadata[u'Requires-Dist']
        return spec and filter(None, spec.splitlines())

    def _read_egg_info_reqs(self):
        source = self.read_text(u'requires.txt')
        return source and self._deps_from_requires_text(source)

    @classmethod
    def _deps_from_requires_text(cls, source):
        section_pairs = cls._read_sections(source.splitlines())
        sections = {section: list(map(operator.itemgetter(u'line'), results
            )) for section, results in itertools.groupby(section_pairs,
            operator.itemgetter(u'section'))}
        return cls._convert_egg_info_reqs_to_simple_reqs(sections)

    @staticmethod
    def _read_sections(lines):
        section = None
        for line in filter(None, lines):
            section_match = re.match(u'\\[(.*)\\]$', line)
            if section_match:
                section = section_match.group(1)
                continue
            yield locals()

    @staticmethod
    def _convert_egg_info_reqs_to_simple_reqs(sections):
        u"\n        Historically, setuptools would solicit and store 'extra'\n        requirements, including those with environment markers,\n        in separate sections. More modern tools expect each\n        dependency to be defined separately, with any relevant\n        extras and environment markers attached directly to that\n        requirement. This method converts the former to the\n        latter. See _test_deps_from_requires_text for an example.\n        "

        def make_condition(name):
            return name and u'extra == "{name}"'.format(name=name)

        def parse_condition(section):
            section = section or u''
            extra, sep, markers = section.partition(u':')
            if extra and markers:
                markers = u'({markers})'.format(markers=markers)
            conditions = list(filter(None, [markers, make_condition(extra)]))
            return u'; ' + u' and '.join(conditions) if conditions else u''
        for section, deps in sections.items():
            for dep in deps:
                yield dep + parse_condition(section)


class DistributionFinder(MetaPathFinder):
    u'\n    A MetaPathFinder capable of discovering installed distributions.\n    '

    @abc.abstractmethod
    def find_distributions(self, name=None, path=None):
        u'\n        Find distributions.\n\n        Return an iterable of all Distribution instances capable of\n        loading the metadata for packages matching the ``name``\n        (or all names if not supplied) along the paths in the list\n        of directories ``path`` (defaults to sys.path).\n        '


@install
class MetadataPathFinder(NullFinder, DistributionFinder):
    u'A degenerate finder for distribution packages on the file system.\n\n    This finder supplies only a find_distributions() method for versions\n    of Python that do not have a PathFinder find_distributions().\n    '

    def find_distributions(self, name=None, path=None):
        u'\n        Find distributions.\n\n        Return an iterable of all Distribution instances capable of\n        loading the metadata for packages matching the ``name``\n        (or all names if not supplied) along the paths in the list\n        of directories ``path`` (defaults to sys.path).\n        '
        if path is None:
            path = sys.path
        pattern = u'.*' if name is None else re.escape(name)
        found = self._search_paths(pattern, path)
        return map(PathDistribution, found)

    @classmethod
    def _search_paths(cls, pattern, paths):
        u'Find metadata directories in paths heuristically.'
        return itertools.chain.from_iterable(cls._search_path(path, pattern
            ) for path in map(cls._switch_path, paths))

    @staticmethod
    def _switch_path(path):
        if not PYPY_OPEN_BUG or os.path.isfile(path):
            with suppress(Exception):
                return zipp.Path(path)
        return pathlib.Path(path)

    @classmethod
    def _matches_info(cls, normalized, item):
        template = u'{pattern}(-.*)?\\.(dist|egg)-info'
        manifest = template.format(pattern=normalized)
        return re.match(manifest, item.name, flags=re.IGNORECASE)

    @classmethod
    def _matches_legacy(cls, normalized, item):
        template = u'{pattern}-.*\\.egg[\\\\/]EGG-INFO'
        manifest = template.format(pattern=normalized)
        return re.search(manifest, str(item), flags=re.IGNORECASE)

    @classmethod
    def _search_path(cls, root, pattern):
        if not root.is_dir():
            return ()
        normalized = pattern.replace(u'-', u'_')
        return (item for item in root.iterdir() if cls._matches_info(
            normalized, item) or cls._matches_legacy(normalized, item))


class PathDistribution(Distribution):

    def __init__(self, path):
        u'Construct a distribution from a path to the metadata directory.'
        self._path = path

    def read_text(self, filename):
        with suppress(FileNotFoundError, IsADirectoryError, KeyError,
            NotADirectoryError, PermissionError):
            return self._path.joinpath(filename).read_text(encoding=u'utf-8')
    read_text.__doc__ = Distribution.read_text.__doc__

    def locate_file(self, path):
        return self._path.parent / path


def distribution(package):
    u'Get the ``Distribution`` instance for the given package.\n\n    :param package: The name of the package as a string.\n    :return: A ``Distribution`` instance (or subclass thereof).\n    '
    return Distribution.from_name(package)


def distributions():
    u'Get all ``Distribution`` instances in the current environment.\n\n    :return: An iterable of ``Distribution`` instances.\n    '
    return Distribution.discover()


def metadata(package):
    u'Get the metadata for the package.\n\n    :param package: The name of the distribution package to query.\n    :return: An email.Message containing the parsed metadata.\n    '
    return Distribution.from_name(package).metadata


def version(package):
    u'Get the version string for the named package.\n\n    :param package: The name of the distribution package to query.\n    :return: The version string for the package as defined in the package\'s\n        "Version" metadata key.\n    '
    return distribution(package).version


def entry_points():
    u'Return EntryPoint objects for all installed packages.\n\n    :return: EntryPoint objects for all installed packages.\n    '
    eps = itertools.chain.from_iterable(dist.entry_points for dist in
        distributions())
    by_group = operator.attrgetter(u'group')
    ordered = sorted(eps, key=by_group)
    grouped = itertools.groupby(ordered, by_group)
    return {group: tuple(eps) for group, eps in grouped}


def files(package):
    return distribution(package).files


def requires(package):
    u'\n    Return a list of requirements for the indicated distribution.\n\n    :return: An iterator of requirements, suitable for\n    packaging.requirement.Requirement.\n    '
    return distribution(package).requires


__version__ = version(__name__)
